"use strict";
/*
 * Copyright 2014 Per Eckerdal
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.indent = exports.prettyBreadcrumb = exports.prettyTimeout = exports.prettyError = exports.prettyErrorLocation = void 0;
const chalk = require("chalk");
const ERROR_LOCATIONS = {
    afterHook: 'after hook',
    beforeHook: 'before hook',
};
const COLORS = {
    errorPlace: chalk.cyan,
    errorHeader: chalk.red,
    trace: chalk.grey,
};
/**
 * Generates a (non-colored) human-readable string of where an error occured.
 * The input format is designed so that you can simply pass in a message of type
 * 'error' that was passed to a reporter.
 */
function prettyErrorLocation(errLoc) {
    var _a;
    if (!errLoc) {
        return 'Unknown location';
    }
    else if (errLoc.in === 'uncaught') {
        return 'Uncaught error';
    }
    const placeType = (_a = ERROR_LOCATIONS[errLoc.in]) !== null && _a !== void 0 ? _a : errLoc.in;
    const placeName = 'inName' in errLoc && errLoc.inName ? ` "${errLoc.inName}"` : '';
    return `In ${placeType}${placeName}`;
}
exports.prettyErrorLocation = prettyErrorLocation;
function dedentTraceLine(line) {
    return line.replace(/^  /, '');
}
/**
 * Generates a colored, non-indented, human readable description of a single
 * error, in the format it is given to reporters as a message of type 'error'.
 *
 * @param errorMessage The error message, as given to the reporter:
 *     {
 *       [in: test|beforeHook|afterHook|uncaught]
 *       stack: [error.stack]
 *     }
 */
function prettyError(errorMessage) {
    if (!errorMessage) {
        return '[No error]';
    }
    let result = '';
    const errorLines = errorMessage.stack ? errorMessage.stack.split('\n') : [];
    if (errorMessage.in) {
        result += COLORS.errorPlace(`${prettyErrorLocation(errorMessage)}:`) + ' ';
    }
    result += COLORS.errorHeader(errorLines[0]) + '\n';
    return errorLines
        .slice(1)
        .map(dedentTraceLine)
        .reduce((result, line) => result + COLORS.trace(line) + '\n', result);
}
exports.prettyError = prettyError;
/**
 * Generates a colored, non-indented, human readable description of a test
 * timeout, including when it happened.
 *
 * @param location Information about what part of the test timed out.
 *     The object should be of the form that prettyErrorLocation expects.
 */
function prettyTimeout(errLoc) {
    return (COLORS.errorPlace(`${prettyErrorLocation(errLoc)}:`) +
        ' ' +
        COLORS.errorHeader('Timed out') +
        '\n');
}
exports.prettyTimeout = prettyTimeout;
function prettyBreadcrumb(breadcrumb, place) {
    const result = `${(place ? COLORS.errorPlace(`${place}: `) : '') + breadcrumb.message}\n`;
    if (breadcrumb.trace) {
        return breadcrumb.trace
            .split(/\n/)
            .map(dedentTraceLine)
            .reduce((result, line) => result + COLORS.trace(line) + '\n', result);
    }
    return result;
}
exports.prettyBreadcrumb = prettyBreadcrumb;
function multiplyString(str, num) {
    return typeof num === 'number' && num > 0 ? str + multiplyString(str, num - 1) : '';
}
function spaces(num) {
    return multiplyString(' ', num);
}
/**
 * Takes a string (possibly with newlines in it) and indents it with a given
 * number of spaces. Blank lines (for example the last line after an ending
 * newline) are not indented.
 *
 * @param str The string to indent
 * @param num The number of spaces to indent with
 */
function indent(str, num) {
    const space = spaces(num);
    return str
        .split('\n')
        .map((line) => (line.match(/^\s*$/) ? line : space + line))
        .join('\n');
}
exports.indent = indent;
