"use strict";
/*
 * Copyright 2014 Per Eckerdal
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _InsertionLog_messages;
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * InsertionLog is a class that helps with printing to the console. The thing it
 * adds over console.log is that it is capable of inserting lines between and
 * replacing already printed lines.
 *
 * InsertionLog will only work if it gets exclusive access to its output stream.
 * If someone else writes to it, it will cause InsertionLog to do the wrong
 * thing. It is ok for someone else to take over after InsertionLog, but then
 * it can't be used anymore.
 */
class InsertionLog {
    constructor(_stream) {
        this._stream = _stream;
        _InsertionLog_messages.set(this, []);
    }
    cursorUp(n) {
        if (n > 0) {
            this._stream.write(`\u001b[${n}A`);
        }
    }
    deleteLineAndGoToBeginning() {
        this._stream.write(`\u001b[2K\u001b[0G`);
    }
    getLastMessages(numMessages) {
        return __classPrivateFieldGet(this, _InsertionLog_messages, "f").slice(__classPrivateFieldGet(this, _InsertionLog_messages, "f").length - numMessages);
    }
    rewindMessages(numMessages) {
        const numLines = this.getLastMessages(numMessages).reduce((lines, message) => lines + 1 + (message.contents.match(/\n/g) || []).length, 0);
        this.cursorUp(numLines);
    }
    printMessages(numMessages) {
        this.getLastMessages(numMessages).forEach((message) => message.contents.split(/\n/).forEach((line) => {
            // We need to make sure to clear every line that we write to, otherwise
            // there may be traces left of what was there before.
            this.deleteLineAndGoToBeginning();
            this._stream.write(`${line}\n`);
        }));
    }
    indexOfMessage(messageId) {
        const idx = __classPrivateFieldGet(this, _InsertionLog_messages, "f").reduce((idx, msg, i) => (msg.id === messageId ? i : idx), -1);
        if (idx === -1) {
            throw new Error(`No message found with id ${messageId}`);
        }
        return idx;
    }
    logAt(messageIdx, message, messageId) {
        __classPrivateFieldSet(this, _InsertionLog_messages, [
            ...__classPrivateFieldGet(this, _InsertionLog_messages, "f").slice(0, messageIdx),
            { contents: message, id: messageId },
            ...__classPrivateFieldGet(this, _InsertionLog_messages, "f").slice(messageIdx),
        ], "f");
        const messagesBack = __classPrivateFieldGet(this, _InsertionLog_messages, "f").length - messageIdx;
        this.rewindMessages(messagesBack - 1);
        this.printMessages(messagesBack);
    }
    /**
     * Append a message to the end of the log.
     *
     * @param message String. The message to write (should not contain a newline
     *     char).
     * @param messageId String. Optional (defaults to null). Id of the message to
     *     write. If you want to be able to overwrite it or insert messages before
     *     or after this message you need to assign an id to it.
     */
    log(message, messageId) {
        __classPrivateFieldGet(this, _InsertionLog_messages, "f").push({ id: messageId, contents: message });
        this.printMessages(1);
    }
    /**
     * Insert a new message after a given log message.
     *
     * @param afterMessageId The id of the message to write after. If there are
     *     duplicates, the last message with the given id will be chosen.
     * @param message String. The message to insert (should not contain a newline
     *     char).
     * @param messageId String. Optional (defaults to null). Id of the message to
     *     write. If you want to be able to overwrite it or insert messages before
     *     or after this message you need to assign an id to it.
     */
    logAfter(afterMessageId, message, messageId) {
        const messageIdx = this.indexOfMessage(afterMessageId);
        this.logAt(messageIdx + 1, message, messageId);
    }
    /**
     * Insert a new message before a given log message.
     *
     * @param beforeMessageId The id of the message to write before. If there are
     *     duplicates, the last message with the given id will be chosen.
     * @param message String. The message to insert (should not contain a newline
     *     char).
     * @param messageId String. Optional (defaults to null). Id of the message to
     *     write. If you want to be able to overwrite it or insert messages before
     *     or after this message you need to assign an id to it.
     */
    logBefore(beforeMessageId, message, messageId) {
        const messageIdx = this.indexOfMessage(beforeMessageId);
        this.logAt(messageIdx, message, messageId);
    }
    /**
     * Replace an already written log message.
     *
     * @warning It is not allowed to replace a message with one that has a different
     *     number of newlines.
     *
     * @param replacedMessageId The id of the message to replace. If there are
     *     duplicates, the last message with the given id will be chosen.
     * @param message String. The message to insert (should not contain a newline
     *     char)
     */
    replace(replacedMessageId, message) {
        const messageIdx = this.indexOfMessage(replacedMessageId);
        const messagesBack = __classPrivateFieldGet(this, _InsertionLog_messages, "f").length - messageIdx;
        this.rewindMessages(messagesBack);
        __classPrivateFieldGet(this, _InsertionLog_messages, "f")[messageIdx].contents = message;
        this.printMessages(messagesBack);
    }
}
exports.default = InsertionLog;
_InsertionLog_messages = new WeakMap();
