"use strict";
/*
 * Copyright 2014 Per Eckerdal
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const bdd_mocha_context_1 = require("./bdd_mocha_context");
const moduleCache = {};
const testInterface = (_, file, runtimeContext) => {
    const absoluteFile = path.resolve(file);
    if (absoluteFile in moduleCache) {
        return moduleCache[absoluteFile];
    }
    const context = new bdd_mocha_context_1.Context(runtimeContext);
    global.context = context;
    let suiteContext;
    let suite = { type: 'suite', contents: [] };
    function hookHandler(type) {
        return (nameOrFn, fn) => {
            var _a;
            let name;
            if (typeof nameOrFn === 'string') {
                name = nameOrFn;
            }
            else {
                fn = nameOrFn;
                name = fn.name ? fn.name : undefined;
            }
            if (typeof fn !== 'function') {
                throw new Error(`Got invalid hook function ${fn}`);
            }
            const run = fn.bind(context);
            suite[type] = [...((_a = suite[type]) !== null && _a !== void 0 ? _a : []), name ? { name, run } : { run }];
        };
    }
    function testForDuplicate(type, name) {
        var _a;
        if ((_a = suite.contents) === null || _a === void 0 ? void 0 : _a.find((test) => test.name === name)) {
            throw new Error(`Redefining ${type} "${name}"`);
        }
    }
    // Since we always run only one test per process, there is no difference between
    // a hook that runs before every test and a hook that runs before a test suite.
    global.before = global.beforeEach = hookHandler('before');
    global.after = global.afterEach = hookHandler('after');
    function describe(options, name, attributesOrFn, fn) {
        testForDuplicate('suite', name);
        let attributes = undefined;
        if (typeof attributesOrFn === 'function') {
            fn = attributesOrFn;
        }
        else {
            attributes = attributesOrFn;
        }
        const [parentSuite, parentContext] = [suite, suiteContext];
        const subSuite = Object.assign(Object.assign({ type: 'suite', contents: [] }, options), { name });
        if (!fn) {
            subSuite.skipped = true;
        }
        suite = subSuite;
        if (parentSuite.attributes || attributes) {
            suite.attributes = Object.assign(Object.assign({}, parentSuite.attributes), attributes);
        }
        const ctx = new bdd_mocha_context_1.SuiteContextImpl(suiteContext, suite.attributes);
        suiteContext = ctx;
        if (fn) {
            fn.apply(ctx);
        }
        suiteContext = parentContext;
        suite = parentSuite;
        const { contents = [] } = suite;
        suite.contents = [...contents, Object.assign(Object.assign({}, ctx.getParameters()), subSuite)];
    }
    global.describe = Object.assign(describe.bind(this, {}), {
        skip: describe.bind(this, { skipped: true }),
        only: describe.bind(this, { only: true }),
        unstable: describe.bind(this, { unstable: true }),
    });
    function it(options, name, attributesOrFn, fn) {
        testForDuplicate('test', name);
        let attributes = undefined;
        if (typeof attributesOrFn === 'function') {
            fn = attributesOrFn;
        }
        else {
            attributes = attributesOrFn;
        }
        const test = Object.assign(Object.assign({ type: 'test' }, options), { name });
        if (!fn) {
            test.skipped = true;
        }
        if (suite.attributes || attributes) {
            test.attributes = Object.assign(Object.assign({}, suite.attributes), attributes);
        }
        if (fn) {
            test.run = fn.bind(context);
        }
        const { contents = [] } = suite;
        suite.contents = [...contents, test];
    }
    global.it = Object.assign(it.bind(this, {}), {
        skip: it.bind(this, { skipped: true }),
        only: it.bind(this, { only: true }),
        unstable: it.bind(this, { unstable: true }),
    });
    require(absoluteFile);
    moduleCache[absoluteFile] = suite;
    return suite;
};
exports.default = testInterface;
