"use strict";
/*
 * Copyright 2014-2016 Per Eckerdal
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.listTestsOfFile = exports.ListTestError = void 0;
/**
 * This file is a node module that exposes functionality to list the tests
 * that a given test suite has. It uses the bin/list_suite script to do this.
 */
const childProcess = require("child_process");
const promises_1 = require("timers/promises");
function streamToString(stream) {
    return new Promise((resolve) => {
        let string = '';
        stream.on('data', (data) => (string += data));
        stream.on('end', () => resolve(string));
    });
}
function withTimeout(promise, timeout, onTimeout) {
    if (timeout === 0) {
        return promise;
    }
    const ac = new AbortController();
    const { signal } = ac;
    const abort = () => ac.abort();
    return Promise.race([(0, promises_1.setTimeout)(timeout, 0, { signal }).then(onTimeout), promise.finally(abort)]);
}
/**
 * An error "class" that means that an error occured when listing the tests of
 * a suite.
 */
class ListTestError extends Error {
    constructor(message, errorOutput, timeout = false) {
        super(message);
        this.timeout = timeout;
        this.stack = this.message + (errorOutput ? ':\n' + errorOutput : '');
    }
}
exports.ListTestError = ListTestError;
function listTestsOfFile(timeout, testInterfacePath, testInterfaceParameter = '', suite, options = {}) {
    var _a;
    const fork = (_a = options.fork) !== null && _a !== void 0 ? _a : childProcess.fork;
    const args = [testInterfacePath, testInterfaceParameter, suite];
    const child = fork(`${__dirname}/bin/list_suite`, args, { silent: true });
    const successObjectPromise = streamToString(child.stdout)
        .then((stdoutStr) => JSON.parse(stdoutStr))
        .catch(() => []);
    const stderrPromise = streamToString(child.stderr);
    const resultPromise = new Promise((resolve) => child.on('exit', resolve)).then((code) => code === 0
        ? successObjectPromise
        : stderrPromise.then((stderrStr) => {
            throw new ListTestError(`Failed to process ${suite}`, stderrStr);
        }));
    return withTimeout(resultPromise, timeout, () => {
        child.kill('SIGKILL');
        throw new ListTestError(`Timed out while listing tests of ${suite}`, undefined, true);
    });
}
exports.listTestsOfFile = listTestsOfFile;
