"use strict";
/*
 * Copyright 2014 Per Eckerdal
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const readline = require("readline");
const through = require("through");
const insertion_log_1 = require("../insertion_log");
const test_path_util_1 = require("../test_path_util");
function spacesForPath(path) {
    return ' '.repeat(path.path.length * 2);
}
const SYMBOLS = {
    inProgress: ' ',
    success: '✓',
    failure: '✖',
    timeout: '⧖',
    skipped: '-',
    aborted: '-',
};
const SYMBOL_COLORS = {
    inProgress: chalk.grey,
    success: chalk.green,
    failure: chalk.red,
    timeout: chalk.red,
    skipped: chalk.cyan,
    aborted: chalk.yellow,
};
const NAME_COLORS = {
    skipped: chalk.cyan,
    failure: chalk.red,
    timeout: chalk.red,
    aborted: chalk.yellow,
    defaultColor: chalk.grey,
};
const SLOWNESS_COLORS = {
    slow: chalk.red,
    halfSlow: chalk.yellow,
};
function color(palette, name) {
    var _a, _b;
    return (_b = (_a = palette[name]) !== null && _a !== void 0 ? _a : palette.defaultColor) !== null && _b !== void 0 ? _b : ((x) => x);
}
function getLine(testPath, unstable, result, breadcrumb) {
    var _a;
    const name = testPath.path.at(-1) + (unstable ? ' [unstable]' : '');
    const status = result !== null && result !== void 0 ? result : 'inProgress';
    const symbolColor = color(SYMBOL_COLORS, status);
    const sign = (_a = SYMBOLS[status]) !== null && _a !== void 0 ? _a : '?';
    const nameColor = color(NAME_COLORS, status);
    breadcrumb = breadcrumb ? `  >  ${breadcrumb}` : '';
    return `${spacesForPath(testPath)}${symbolColor(sign)} ${nameColor(name)}${breadcrumb}`;
}
/**
 * SpecProgress is a reporter that emits the progress of tests while they run
 * to a given stream. It does not print any test output other than the list of
 * test names and their results coded as checkmarks or crosses. It does not
 * print a summary of how the tests went at the end; for this reason it is
 * typically used together with some other summary reporter.
 *
 * SpecProgress uses InsertionLog to print progress of tests as they run in
 * parallel. This means that the output will be borked unless the stream is
 * printed to a terminal that supports overwriting already written text. If
 * the output does not support this, the Serializer reporter can be used to
 * print tests as if they were run serially.
 */
class SpecProgress {
    constructor(streams, insertionLogFactory) {
        // The reporter needs to be able to insert new lines for tests and place
        // them where they belong. This is done by keeping track of ids for the
        // last line that was inserted for each test suite.
        this._lastLineIdForSuite = new Map();
        this._isUnstable = new Set();
        this._testStartTime = new Map();
        this._stdio = new Map();
        const { stdout, disableBreadcrumbs = false } = streams !== null && streams !== void 0 ? streams : {};
        this._log = insertionLogFactory ? insertionLogFactory(stdout) : new insertion_log_1.default(stdout);
        this._disableBreadcrumbs = disableBreadcrumbs;
    }
    makePipedStream(id) {
        const input = through();
        const lines = readline.createInterface({ input, output: through() });
        const idForCounter = (counter) => id + (counter ? `_${counter}` : '');
        let counter = 0;
        lines.on('line', (line) => {
            this._log.logAfter(idForCounter(counter), line, idForCounter(counter + 1));
            counter++;
        });
        return input;
    }
    gotMessage(testPath, message, time) {
        var _a, _b, _c;
        const pathAsString = JSON.stringify(testPath);
        if (message.type === 'suiteStart') {
            const suitePath = message.suite;
            const suitePathString = JSON.stringify(suitePath);
            const suiteName = suitePath.path.at(-1) || '';
            this._log.log(spacesForPath(suitePath) + suiteName, suitePathString);
            this._lastLineIdForSuite.set(suitePathString, suitePathString);
        }
        else if (message.type === 'breadcrumb' && testPath && !this._disableBreadcrumbs) {
            const elapsedMs = time.getTime() - ((_a = this._testStartTime.get(pathAsString)) !== null && _a !== void 0 ? _a : 0);
            const line = getLine(testPath, this._isUnstable.has(pathAsString), undefined, `[${(elapsedMs / 1000).toFixed(1)}s]: ${message.message}`);
            this._log.replace(pathAsString, line);
        }
        else if (message.type === 'stdout') {
            (_b = this._stdio.get(pathAsString)) === null || _b === void 0 ? void 0 : _b.stdout.write(message.data);
        }
        else if (message.type === 'stderr') {
            (_c = this._stdio.get(pathAsString)) === null || _c === void 0 ? void 0 : _c.stderr.write(message.data);
        }
        else if (message.type === 'start' || message.type === 'finish') {
            const suitePathAsString = JSON.stringify((0, test_path_util_1.suitePathOf)(testPath));
            if (message.type === 'start') {
                const line = getLine(testPath, message.unstable);
                const lineId = this._lastLineIdForSuite.get(suitePathAsString);
                lineId && this._log.logAfter(lineId, line, pathAsString);
                this._lastLineIdForSuite.set(suitePathAsString, pathAsString);
                message.unstable
                    ? this._isUnstable.add(pathAsString)
                    : this._isUnstable.delete(pathAsString);
                this._stdio.set(pathAsString, {
                    stdout: this.makePipedStream(pathAsString),
                    stderr: this.makePipedStream(pathAsString),
                });
            }
            else if (message.type === 'finish') {
                let line = getLine(testPath, message.unstable, message.result);
                if (message.duration && (message.slow || message.halfSlow)) {
                    const slownessColor = SLOWNESS_COLORS[message.slow ? 'slow' : 'halfSlow'];
                    line += slownessColor(' (' + Math.round(message.duration) + 'ms)');
                }
                this._log.replace(pathAsString, line);
            }
        }
        else if (message.type === 'startedTest') {
            this._testStartTime.set(pathAsString, time.getTime());
        }
    }
}
exports.default = SpecProgress;
