"use strict";
/*
 * Copyright 2014 Per Eckerdal
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const combined_1 = require("./combined");
/**
 * Timer is a reporter for internal use by the suite runner. It forwards all
 * messages to another reporter, but also adds "time", "slow" and "halfSlow"
 * fields to finish messages.
 *
 * This is used in the suite runner as a way to move logic out of that file.
 */
class Timer extends combined_1.default {
    constructor(reporter) {
        super([reporter]);
        this._slowThresholdOverrides = new Map();
        this._testStartTime = new Map();
        this._testDoneTime = new Map();
    }
    getSlowThresholdForTest(key) {
        var _a, _b;
        return (_b = (_a = this._slowThresholdOverrides.get(key)) !== null && _a !== void 0 ? _a : this._defaultSlowThreshold) !== null && _b !== void 0 ? _b : 0;
    }
    setSlowThresholdForTest(key, value) {
        this._slowThresholdOverrides.set(key, value);
    }
    registerTests(tests, options, time) {
        super.registerTests(tests, options, time);
        this._defaultSlowThreshold = options.slowThreshold;
    }
    gotMessage(testPath, message, time) {
        let messageWithSlowness;
        const key = JSON.stringify(testPath);
        if (message.type === 'finish') {
            const [done, start] = [this._testDoneTime.get(key), this._testStartTime.get(key)];
            let duration;
            let [slow, halfSlow] = [false, false];
            if (done && start) {
                duration = done - start;
                const slowThreshold = this.getSlowThresholdForTest(key);
                [slow, halfSlow] = [duration >= slowThreshold, duration >= slowThreshold / 2];
            }
            messageWithSlowness = Object.assign(Object.assign({}, message), { duration, slow, halfSlow });
        }
        else {
            if (message.type === 'startedTest') {
                this._testStartTime.set(key, time.getTime());
            }
            else if (message.type === 'startedAfterHooks') {
                this._testDoneTime.set(key, time.getTime());
            }
            else if (message.type === 'setSlowThreshold') {
                this.setSlowThresholdForTest(key, message.value);
            }
            messageWithSlowness = message;
        }
        super.gotMessage(testPath, messageWithSlowness, time);
    }
}
exports.default = Timer;
