"use strict";
/*
 * Copyright 2014 Per Eckerdal
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const async = require("async");
const childProcess = require("child_process");
const chai_1 = require("chai");
const path = require("path");
const list_suite_1 = require("./list_suite");
const soft_kill_1 = require("./soft_kill");
const timestamper_1 = require("./reporters/timestamper");
const combined_1 = require("./reporters/combined");
const cancellable_1 = require("./reporters/cancellable");
const error_detector_1 = require("./reporters/error_detector");
const spec_1 = require("./reporters/spec");
const errorMessageUtil = require("./error_message_util");
const test_failure_error_1 = require("./test_failure_error");
const timeout_timer_1 = require("./timeout_timer");
const defaultInterface = `${__dirname}/interfaces/bdd_mocha`;
/**
 * Types used in this file:
 * * Test path
 * * Suite descriptor
 */
function reportSkippedTest(reporter, testPath, unstable) {
    reporter.gotMessage(testPath, { type: 'start', skipped: true, unstable: unstable });
    reporter.gotMessage(testPath, { type: 'finish', result: 'skipped', unstable: unstable });
}
function hookStream(testPath, stream, reporter, type) {
    const onData = (data) => reporter.gotMessage(testPath, { type, data });
    stream.on('data', (data) => onData(data.toString()));
}
function runTestProcess(childProcess, debugPort, timeout, slowThreshold, reporter, testInterfacePath, interfaceParameter, testPath, killSubProcesses, attributes) {
    const parameters = { timeout, slowThreshold, interfaceParameter, killSubProcesses, attributes };
    const debugArgs = debugPort ? ['--inspect-brk=' + debugPort] : [];
    const child = childProcess.fork(`${__dirname}/bin/run_test`, [testInterfacePath, JSON.stringify(parameters), testPath.file, ...testPath.path], { silent: true, execArgv: [...process.execArgv, ...debugArgs] });
    // might be null in case of listening for debugger
    if (child.stdout && child.stderr) {
        hookStream(testPath, child.stdout, reporter, 'stdout');
        hookStream(testPath, child.stderr, reporter, 'stderr');
    }
    return child;
}
function killAfter(softKill, timerFactory, child, timeToWait, graceTime, willKill) {
    const timer = timerFactory(timeToWait);
    timer.on('timeout', () => {
        willKill();
        softKill(child, graceTime);
    });
    child.on('exit', () => timer.cancel());
    child.on('message', (message) => {
        if (message.type === 'setTimeout' && typeof message.value === 'number') {
            timer.updateTimeout(message.value);
        }
    });
}
function runTest(softKill, timerFactory, childProcess, debugPort, timeout, graceTime, slowThreshold, reporter, testInterfacePath, testInterfaceParameter, testPath, unstable, killSubProcesses, attributes) {
    let didTimeout = false;
    const child = runTestProcess(childProcess, debugPort, timeout, slowThreshold, reporter, testInterfacePath, testInterfaceParameter, testPath, killSubProcesses, attributes);
    child.on('message', (message) => reporter.gotMessage(testPath, message));
    if (timeout !== 0) {
        killAfter(softKill, timerFactory, child, timeout, graceTime, () => {
            didTimeout = true;
            reporter.gotMessage(testPath, { type: 'timeout' });
        });
    }
    const closePromise = new Promise((resolve) => child.on('close', () => resolve()));
    return new Promise((resolve, reject) => {
        child.on('exit', (code, signal) => {
            // The 'close' event is typically emitted *after* the 'exit' event. This means
            // that when we get to this point, there may still be outstanding messages in
            // flight from the child process. Because of that, we need to wait for 'close'.
            closePromise.then(() => {
                if (didTimeout) {
                    reporter.gotMessage(testPath, { type: 'finish', result: 'timeout', unstable });
                    reject(new Error('Test timed out'));
                }
                else {
                    reporter.gotMessage(testPath, {
                        type: 'finish',
                        result: code === 0 ? 'success' : 'failure',
                        unstable: unstable,
                        code: code !== null && code !== void 0 ? code : undefined,
                        signal: signal !== null && signal !== void 0 ? signal : undefined,
                    });
                    if (code === 0) {
                        resolve();
                    }
                    else if (signal) {
                        reject(new Error(`Test process exited with signal ${signal}`));
                    }
                    else {
                        reject(new Error(`Test process exited with non-zero exit code ${code}`));
                    }
                }
            }, reject);
        });
    });
}
function runTestWithRetries(softKill, timerFactory, childProcess, debugPort, timeout, graceTime, attempts, slowThreshold, reporter, testInterfacePath, testInterfaceParameter, testPath, unstable, killSubProcesses, attributes) {
    // When tests are retried, we need to "lie" about the finish message; tests
    // that are retried aren't actually finished, so they should not be reported
    // as such.
    class RetryingReporter {
        constructor(inner) {
            this.inner = inner;
        }
        registerTests(tests, options) {
            this.inner.registerTests(tests, options);
        }
        registrationFailed(error) {
            this.inner.registrationFailed(error);
        }
        gotMessage(testPath, message) {
            if (message.type === 'finish') {
                this._postponedFinishMessage = message;
            }
            else {
                this.inner.gotMessage(testPath, message);
            }
        }
        done() {
            this.inner.done();
        }
        sendPostponsedFinishMessage(changeToRetry) {
            (0, chai_1.expect)(this._postponedFinishMessage).not.to.be.undefined;
            const message = changeToRetry
                ? Object.assign(Object.assign({}, this._postponedFinishMessage), { type: 'retry' })
                : this._postponedFinishMessage;
            this.inner.gotMessage(testPath, message);
        }
    }
    const retryingReporter = new RetryingReporter(reporter);
    return runTest(softKill, timerFactory, childProcess, debugPort, timeout, graceTime, slowThreshold, retryingReporter, testInterfacePath, testInterfaceParameter, testPath, unstable, killSubProcesses, attributes)
        .then(() => retryingReporter.sendPostponsedFinishMessage())
        .catch((error) => {
        retryingReporter.sendPostponsedFinishMessage(attempts > 1);
        if (attempts <= 1) {
            throw error;
        }
        return runTestWithRetries(softKill, timerFactory, childProcess, debugPort, timeout, graceTime, attempts - 1, slowThreshold, reporter, testInterfacePath, testInterfaceParameter, testPath, unstable, killSubProcesses, attributes);
    });
}
function runOrSkipTest(softKill, timerFactory, childProcess, debugPort, attempts, globalTimeout, graceTime, globalSlowThreshold, reporter, testInterfacePath, testInterfaceParameter, testDescriptor, runUnstable, killSubProcesses) {
    return __awaiter(this, void 0, void 0, function* () {
        const testPath = testDescriptor.path;
        const testAttributes = testDescriptor.attributes;
        const unstable = !!testDescriptor.unstable;
        if (testDescriptor.skipped || (unstable && !runUnstable)) {
            reportSkippedTest(reporter, testPath, unstable);
            return;
        }
        reporter.gotMessage(testPath, { type: 'start', unstable });
        if (testAttributes) {
            reporter.gotMessage(testPath, { type: 'attributes', attributes: testAttributes });
        }
        const timeout = typeof testDescriptor.timeout === 'number' ? testDescriptor.timeout : globalTimeout;
        const slowThreshold = typeof testDescriptor.slow === 'number' ? testDescriptor.slow : globalSlowThreshold;
        yield runTestWithRetries(softKill, timerFactory, childProcess, debugPort, timeout, graceTime, attempts, slowThreshold, reporter, testInterfacePath, testInterfaceParameter, testPath, unstable, killSubProcesses, testAttributes);
    });
}
/**
 * Takes a list of tests. If none of the tests are marked as "only", then returns
 * the entire list. Otherwise returns only the tests that are marked as "only".
 */
function filterOnly(disallowOnly, tests) {
    const onlyTests = tests.filter((test) => test.only);
    if (disallowOnly && onlyTests.length !== 0) {
        throw new Error('Encountered tests marked as .only, and the disallowOnly flag is set');
    }
    return onlyTests.length === 0 ? tests : onlyTests;
}
function filterAttributes(attributeFilter, tests) {
    if (!attributeFilter) {
        return tests;
    }
    return tests.filter((test) => {
        const attributes = test.attributes || {};
        try {
            return attributeFilter(attributes);
        }
        catch (error) {
            if (error instanceof Error) {
                error.message = `Encountered error while filtering attributes: ${error.message}`;
            }
            throw error;
        }
    });
}
function filterTest(testFilter, tests) {
    if (!testFilter) {
        return tests;
    }
    return tests.filter((test) => {
        try {
            return testFilter(test);
        }
        catch (error) {
            if (error instanceof Error) {
                error.message = 'Encountered error while filtering tests: ' + error.message;
            }
            throw error;
        }
    });
}
function filterGrep(grep, invertGrep, tests) {
    if (!grep) {
        return tests;
    }
    return tests.filter((test) => {
        const match = test.path.path.join(' ').match(grep);
        return invertGrep ? !match : match;
    });
}
function listTestsOfFiles(timeout, testInterfacePath, testInterfaceParameter, files) {
    return __awaiter(this, void 0, void 0, function* () {
        const fileTests = yield Promise.all(files.map((file) => (0, list_suite_1.listTestsOfFile)(timeout, testInterfacePath, testInterfaceParameter, file)));
        return fileTests.flat();
    });
}
function listTests(reporter, timeout, testInterfacePath, testInterfaceParameter, files) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield listTestsOfFiles(timeout, testInterfacePath, testInterfaceParameter, files);
        }
        catch (error) {
            if (!(error instanceof list_suite_1.ListTestError)) {
                throw error;
            }
            reporter.registrationFailed(error);
            throw new test_failure_error_1.default(error.timeout ? error.message : 'Failed to process test files');
        }
    });
}
/**
 * Run a set of test suites. For information about what options are supported,
 * please refer to the documentation in doc/suite_runner_api.md.
 *
 * Parameters for internal use only:
 * childProcess: Injecting the child_process module for mocking. Optional and only useful for internal testing.
 * timeoutTimer: Injecting the timeout_timer module for mocking. Optional and only useful for internal testing.
 * softKill: Injecting the soft_kill module for mocking. Optional and only useful for internal testing.
 * clock: Injecting a clock for mocking. Should be a function that returns a Date representing the current time. Optional and only useful for internal testing.
 *
 * Returns a promise that succeeds if no tests fail. If one or more tests fail,
 * the promise will fail with a TestFailureError. If the promise fails with any
 * other type of error, it means that there is a bug, for example in a reporter
 * or in the suite runner.
 */
function default_1(options) {
    var _a, _b, _c, _d, _e, _f, _g;
    return __awaiter(this, void 0, void 0, function* () {
        const timeout = (_a = options.timeout) !== null && _a !== void 0 ? _a : 10000;
        const listingTimeout = (_b = options.listingTimeout) !== null && _b !== void 0 ? _b : 60000;
        const slowThreshold = options.slowThreshold || 1000;
        const graceTime = (_c = options.graceTime) !== null && _c !== void 0 ? _c : 500;
        const attempts = options.attempts || 0;
        const testInterfacePath = path.resolve(options.interface || defaultInterface);
        const testInterfaceParameter = options.interfaceParameter;
        const errorDetector = new error_detector_1.default();
        const clock = (_d = options.clock) !== null && _d !== void 0 ? _d : (() => new Date());
        const reporters = (_e = options.reporters) !== null && _e !== void 0 ? _e : [new spec_1.default(process)];
        const internalErrorOutput = (_f = options.internalErrorOutput) !== null && _f !== void 0 ? _f : process.stderr;
        const reporter = new cancellable_1.default(new timestamper_1.default(new combined_1.default([
            errorDetector,
            ...(Array.isArray(reporters) ? reporters : [reporters]),
        ]), clock));
        if (!Array.isArray(options.files)) {
            throw new Error('Option "files" not present or not Array');
        }
        (_g = options.signal) === null || _g === void 0 ? void 0 : _g.addEventListener('abort', () => {
            if (!reporter.isFinished()) {
                reporter.cancel();
            }
        });
        try {
            let tests = yield listTests(reporter, listingTimeout, testInterfacePath, testInterfaceParameter, options.files);
            tests = filterOnly(options.disallowOnly, tests);
            tests = filterAttributes(options.attributeFilter, tests);
            tests = filterTest(options.testFilter, tests);
            tests = filterGrep(options.grep, options.invertGrep, tests);
            if (options.debugPort) {
                tests = tests.slice(0, 1); // Only run one test when debugging
            }
            if (!tests.length) {
                throw new Error('No tests found');
            }
            reporter.registerTests(tests.map((test) => test.path), { timeout, listingTimeout, slowThreshold, graceTime, attempts });
            yield async.parallelLimit(tests.map((test) => (done) => {
                var _a, _b, _c;
                if (reporter.isFinished()) {
                    done();
                    return;
                }
                return runOrSkipTest((_a = options.softKill) !== null && _a !== void 0 ? _a : soft_kill_1.default, (_b = options.timerFactory) !== null && _b !== void 0 ? _b : ((timeout) => new timeout_timer_1.default(timeout)), (_c = options.childProcess) !== null && _c !== void 0 ? _c : childProcess, options.debugPort, attempts, timeout, graceTime, slowThreshold, reporter, testInterfacePath, testInterfaceParameter, test, options.runUnstable, options.killSubProcesses)
                    .catch(() => { })
                    .finally(() => done());
            }), options.parallelism || 8);
            const cancelled = reporter.isFinished();
            reporter.done();
            if (cancelled) {
                throw new test_failure_error_1.default('Tests cancelled');
            }
            if (errorDetector.didFail()) {
                const testPath = JSON.stringify(errorDetector.testPath());
                const message = JSON.stringify(errorDetector.message());
                throw new test_failure_error_1.default(`Tests failed: testpath: ${testPath} message: ${message}`);
            }
        }
        catch (error) {
            if (!(error instanceof test_failure_error_1.default)) {
                // Test failures will already have been reported by reporters, so there
                // is no need for us to report them here.
                internalErrorOutput.write('Internal error in Overman or a reporter:\n');
                internalErrorOutput.write(`${errorMessageUtil.indent(errorMessageUtil.prettyError(error), 2)}\n`);
            }
            throw error;
        }
    });
}
exports.default = default_1;
